/*
 * Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include "uart.h"

#include <string.h>
#include <stdarg.h>
#include <stdio.h>

#define FRAME_HEAD_BYTE 0xdb
#define FRAME_CTRL_BYTE 0xdc
#define FRAME_TAIL_BYTE 0xde

//#define UART0_BPS         115200
#define UART0_BPS 230400

#if UART0_BPS == 115200
#define UART0_BR0 0x60 //11.0592M / 115200 = 0x0060
#define UART0_BR1 0x00
#elif UART0_BPS == 230400
#define UART0_BR0 0x30 //11.0592M / 230400 = 0x0030
#define UART0_BR1 0x00
#endif

static uint8 buffer_encode[245];
static uint16 buffer_finger[8];

extern uint8 finger_number;
extern uint16 finger_pos_x[];
extern uint16 finger_pos_y[];

const char *gesture_str[] =
	{
		"NO_GESTRUE",			  //0x00
		"ST_SCROLL_UP",			  //0x01
		"ST_SCROLL_RT",			  //0x02
		"ST_SCROLL_DN",			  //0x03
		"ST_SCROLL_LT",			  //0x04
		"ST_FLICK_SLOW_UP",		  //0x05
		"ST_FLICK_SLOW_RT",		  //0x06
		"ST_FLICK_SLOW_DN",		  //0x07
		"ST_FLICK_SLOW_LT",		  //0x08
		"ST_FLICK_FAST_UP",		  //0x09
		"ST_FLICK_FAST_RT",		  //0x0A
		"ST_FLICK_FAST_DN",		  //0x0B
		"ST_FLICK_FAST_LT",		  //0x0C
		"ST_CLICK",				  //0x0D
		"ST_DOUBLECLICK",		  //0x0E
		"UNKNOWN",				  //0x0F
		"MT_UP",				  //0x10
		"MT_RT",				  //0x11
		"MT_DN",				  //0x12
		"MT_LT",				  //0x13
		"ZOOM_IN",				  //0x14
		"ZOOM_OUT",				  //0x15
		"ROTATE_CLOCKWISE",		  //0x16
		"ROTATE_COUNTERCLOCKWISE" //0x17
};

void uart0_init(void)
{
	P3SEL |= 0x30; //P3.4,P3.5 set to UART;

	//P3OUT |= 0x08;         //Reset FT232RL

	UCA0CTL0 = 0x00; //Default value:No parity check, MSB first, 8bits, UART, Asynchronous mode

	UCA0CTL1 |= UCSWRST;  //UART held in reset mode
	UCA0CTL1 |= UCSSEL_1; //ACLK as clock source.

	UCA0BR0 = UART0_BR0;
	UCA0BR1 = UART0_BR1;

	UCA0MCTL &= ~UCOS16;  //disable oversampling modulation
	UCA0CTL1 &= ~UCSWRST; //UART exit reset mode.
}

void uart0_putchar(uint8 ch)
{
	while (!(UC0IFG & UCA0TXIFG))
		;

	UCA0TXBUF = ch;
}

void uart0_puts(char *str)
{
	while (*str != 0)
	{
		uart0_putchar(*str++);
	}
}

void uart0_printf(char *fmt, ...)
{
	va_list ap;
	char string[256];

	va_start(ap, fmt);
	vsprintf(string, fmt, ap);
	uart0_puts(string);
	va_end(ap);
}

void uart1_init(void)
{
	P3SEL |= 0xC0; //P3.6,P3.7 set to UART;

	UCA1CTL0 = 0x00; //Default value:No parity check, MSB first, 8bits, UART, Asynchronous mode

	UCA1CTL1 |= UCSWRST;  //UART held in reset mode
	UCA1CTL1 |= UCSSEL_1; //ACLK as clock source.

	UCA1BR0 = 0x60;
	UCA1BR1 = 0x00; //11.0592M/115200 = 0x60

	UCA1MCTL &= ~UCOS16;  //disable oversampling modulation
	UCA1CTL1 &= ~UCSWRST; //UART exit reset mode.
}

void uart1_putchar(uint8 ch)
{
	while (!(UC1IFG & UCA1TXIFG))
		;

	UCA1TXBUF = ch;
}

void uart1_puts(char *str)
{
	while (*str != 0)
	{
		uart1_putchar(*str++);
	}
}

void uart1_printf(char *fmt, ...)
{
	va_list ap;
	char string[256];

	va_start(ap, fmt);
	vsprintf(string, fmt, ap);
	uart1_puts(string);
	va_end(ap);
}

void send_frame(uint8 size)
{
	uint8 i;

	if (size)
	{
		for (i = 0; i < size; i++)
		{
			uart0_putchar(buffer_encode[i]);
		}
	}
}

uint8 encode_gesture_frame(uint8 gesture_id)
{
	const char *str = NULL;
	uint8 len;

	str = get_gesture_string(gesture_id);
	if (str)
	{
		len = strlen(str);

		return encode_frame(FRAME_LABEL_GESTURE_ID, gesture_id, (uint8 *)str, len);
	}
	else
	{
		return 0;
	}
}

const char *get_gesture_string(uint8 gesture_id)
{
	if (gesture_id <= ROTATE_COUNTERCLOCKWISE)
	{
		return gesture_str[gesture_id];
	}
	else
	{
		return NULL;
	}
}

uint8 encode_calibration_frame(uint16 data)
{
	buffer_finger[0] = data;

	return encode_frame(FRAME_LABEL_CALIBRATION, 0, (uint8 *)buffer_finger, 2);
}

uint8 encode_finger_position_frame(void)
{
	uint8 i;

	for (i = 0; i < 8; i++)
	{
		buffer_finger[i] = 0xFFFF;
	}

	if (finger_number)
	{
		for (i = 0; i < finger_number; i++)
		{
			buffer_finger[2 * i] = finger_pos_x[i];
			buffer_finger[2 * i + 1] = finger_pos_y[i];
		}
	}

	return encode_frame(FRAME_LABEL_FINGER_POS, 0, (uint8 *)buffer_finger, 16);
}

uint8 encode_frame(uint8 label, uint8 extra_label, uint8 *buffer, uint8 size)
{
	uint8 i;
	uint8 buffer_count = 0;

	buffer_encode[buffer_count++] = FRAME_HEAD_BYTE;

	buffer_encode[buffer_count++] = label;

	if (extra_label)
	{
		buffer_encode[buffer_count++] = extra_label;
	}

	for (i = 0; i < size; i++)
	{
		if ((buffer[i] == FRAME_HEAD_BYTE) || (buffer[i] == FRAME_CTRL_BYTE) || (buffer[i] == FRAME_TAIL_BYTE))
		{
			buffer_encode[buffer_count++] = FRAME_CTRL_BYTE;
		}

		buffer_encode[buffer_count++] = buffer[i];
	}

	buffer_encode[buffer_count++] = FRAME_TAIL_BYTE;

	return buffer_count;
}